
## ----prelims,cache=FALSE-------------------------------------------------
library(pomp)
library(plyr)
library(magrittr)
library(foreach)
library(zoo)
library(ggplot2)

## ----load-data-----------------------------------------------------------
data<-read.csv("data22-32.csv",header=T)[,c(2,4)]
covar<-read.csv("covar22-32.csv",header=T)[,-1] 

head(covar)
head(data)

## ----rprocess------------------------------------------------------------
rproc <- Csnippet("
                  double beta, seas, lambda;
                  
                  seas=1+beta1*sin(2*M_PI*(t - phi));   
                  
                  // transmission rate
                  beta = R0*(gamma+mu)*seas;
                  
                  // expected force of infection
                  lambda = beta*(I+iota)/pop;
                  
                  int Mort[30] = {M1,M2,M3,M4,M5,M6,M7,M8,M9,M10,M11,M12,M13,M14,
                  M15,M16,M17,M18,M19,M20,M21,M22,M23,M24,M25,M26,M27,M28,M29,M30};
                  
                  double SumM = 0.0;
                  
                  for(int i = 0; i <= dur; i++){
                  SumM += Mort[i];
                  }
                  
                  S += (births - lambda*(S-((SumM/prod)*(S/pop))) - theta*lambda*(SumM/prod)*(S/pop) - mu*S)*dt;
                  I += (lambda*(S-((SumM/prod)*(S/pop))) + theta*lambda*(SumM/prod)*(S/pop)- gamma*I - mu*I)*dt; 
                  Im += (theta*lambda*(SumM/prod)*(S/pop) - gamma*Im - mu*Im)*dt; 
                  C1 += (alpha1*gamma*(I-Im))*dt;         
                  C2 += (alphaM*gamma*Im)*dt;         
                  ")

## ----initializer---------------------------------------------------------
initlz <- Csnippet("
                   double m = pop/(S_0+I_0+R_0);
                   S = nearbyint(m*S_0);
                   I = nearbyint(m*I_0);
                   Im = Im_0*pop;
                   C1 = 0;
                   C2 = 0;
                   ")

## ----dmeasure------------------------------------------------------------
dmeas <- Csnippet("
                  double m1 = eps1*C1;
                  double m2 = eps2*C2;
                  double v1 = m1*(1.0-eps1+psi1*psi1*m1);
                  double v2 = m2*(1.0-eps2+psi2*psi2*m2);
                  
                  double m = m1 + m2;
                  double v = v1 + v2;
                  double tol = 1.0e-18;
                  if (P > 0.0) {
                  lik = pnorm(P+0.5,m,sqrt(v)+tol,1,0)-pnorm(P-0.5,m,sqrt(v)+tol,1,0)+tol;
                  } else {
                  lik = pnorm(P+0.5,m,sqrt(v)+tol,1,0)+tol;
                  }
                  
                  if (give_log) lik = log(lik);
                  
                  ")

## ----rmeasure------------------------------------------------------------
rmeas <- Csnippet("
                  double m1 = eps1*C1;
                  double m2 = eps2*C2;
                  double v1 = m1*(1.0-eps1+psi1*psi1*m1);
                  double v2 = m2*(1.0-eps2+psi2*psi2*m2);
                  
                  double m = m1 + m2;
                  double v = v1 + v2;
                  double tol = 1.0e-18;
                  P = rnorm(m,sqrt(v)+tol);
                  if (P > 0.0) {
                  P = nearbyint(P);
                  } else {
                  P = 0.0;
                  }
                  ")

## ----transforms----------------------------------------------------------
toEst <- Csnippet("
                  Tmu = log(mu);
                  Tgamma = log(gamma);
                  Talpha1 = logit(alpha1);
                  TalphaM = logit(alphaM);
                  Ttheta = log(theta);
                  Tiota = log(iota);
                  Teps1 = logit(eps1);
                  Teps2 = logit(eps2);
                  Tprod = logit(prod);
                  Tbeta1 = logit(beta1);
                  Tdur = log(dur);
                  Tpsi1 = log(psi1);
                  Tpsi2 = log(psi2);
                  TR0 = log(R0);
                  Tphi = logit(phi);
                  TS_0 =  logit(S_0);
                  TIm_0 =  logit(Im_0);
                  TI_0 =  logit(I_0);
                  TR_0 =  logit(R_0);
                  ")

fromEst <- Csnippet("
                    Tmu = exp(mu);
                    Tgamma = exp(gamma);
                    Talpha1 = expit(alpha1);
                    TalphaM = expit(alphaM);
                    Ttheta = exp(theta);
                    Tbeta1 = expit(beta1);
                    Tiota = exp(iota);
                    Teps1 = expit(eps1);
                    Teps2 = expit(eps2);
                    Tprod = expit(prod);
                    Tdur = exp(dur);
                    Tpsi1 = exp(psi1);
                    Tpsi2 = exp(psi2);
                    TR0 = exp(R0);
                    Tphi = expit(phi);
                    TS_0 =  expit(S_0);
                    TIm_0 =  expit(Im_0);
                    TI_0 =  expit(I_0);
                    TR_0 =  expit(R_0);
                    ")

##--------------- A continuous-time deterministic skeleton ---------------
skel <- Csnippet("
                 double beta, seas, lambda;
                 
                 seas=1+beta1*sin(2*M_PI*(t - phi));   
                 
                 // transmission rate
                 beta = R0*(gamma+mu)*seas;
                 
                 // expected force of infection
                 lambda = beta*(I+iota)/pop;
                 
                 int Mort[30] = {M1,M2,M3,M4,M5,M6,M7,M8,M9,M10,M11,M12,M13,M14,
                 M15,M16,M17,M18,M19,M20,M21,M22,M23,M24,M25,M26,M27,M28,M29,M30};
                 
                 double SumM = 0.0;
                 
                 for(int i = 0; i <= dur; i++){
                 SumM += Mort[i];
                 }
                 
                 
                 DS = births - lambda*(S-((SumM/prod)*(S/pop))) - theta*lambda*(SumM/prod)*(S/pop) - mu*S;
                 DI = lambda*(S-((SumM/prod)*(S/pop))) + theta*lambda*(SumM/prod)*(S/pop) - gamma*I - mu*I; 
                 DIm = theta*lambda*(SumM/prod)*(S/pop) - gamma*Im - mu*Im; 
                 DC1 = alpha1*gamma*(I-Im);
                 DC2 = alphaM*gamma*Im ;
                 ")


## ----pomp-construction---------------------------------------------------
data[30:522,] %>% 
  pomp(t0=with(data[30:522,],2*time[1]-time[2]),
       time="time",toEstimationScale=toEst,
       fromEstimationScale=fromEst,
       rprocess=euler.sim(rproc,delta.t=1/365.25),
       initializer=initlz,
       dmeasure=dmeas,
       rmeasure=rmeas,
       covar=covar[29:522,],
       tcovar="time",
       skeleton=vectorfield(skel), 
       cdir="tmp", 
       zeronames=c("C1","C2"),
       statenames=c("S","Im","I","C1","C2"),
       paramnames=c("R0","mu","gamma","alpha1","iota","alphaM","dur",
                    "prod","eps1","eps2","psi1","psi2","beta1","phi","theta",
                    "S_0","Im_0","I_0","R_0")
  ) -> m1.Det

##### a set of 5000 guesses were defined using Latin Hypercube############
  teta<- c(R0=guess[k,1],beta1=guess[k,2],phi=guess[k,3],iota=guess[k,4],
 dur=guess[k,5],theta=guess[k,6],alpha1=guess[k,7], alphaM=guess[k,8],prod=0.013,
 eps1=0.15,eps2=0.15,psi1=guess[k,9],psi2=guess[k,10],S_0=guess[k,11],
I_0=guess[k,12], Im_0=guess[k,13],R_0=guess[k,14],mu=0.018,gamma=365/21)
  
    
traj.match(m1.Det,start=teta, est=c("R0","beta1","phi","iota","rho","theta","alpha1","alphaM",
"psi1","psi2","S_0","I_0","Im_0","R_0"), maxit=10000,reltol=1e-8,transform=TRUE) %>%
  traj.match(method="SANN",maxit=10000,reltol=1e-8,transform=TRUE)%>% 
  traj.match(method="subplex",maxit=10000,reltol=1e-8,transform=TRUE) ->tm
  
